@@ -6,7 +6,7 @@ use 5.014;
 
 no if $] >= 5.018, warnings => 'experimental::smartmatch';
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use Carp qw(confess cluck);
 use DateTime;
@@ -28,7 +28,8 @@ sub new {
 	my $self = {
 		datetime => $opt{datetime}
 		  // DateTime->now( time_zone => 'Europe/Berlin' ),
-		iris_base => $opt{iris_base}
+		developer_mode => $opt{developer_mode},
+		iris_base      => $opt{iris_base}
 		  // 'http://iris.noncd.db.de/iris-tts/timetable',
 		lookahead => $opt{lookahead} // ( 4 * 60 ),
 		serializable => $opt{serializable},
@@ -96,6 +97,13 @@ sub new {
 	@{ $self->{results} }
 	  = sort { $a->{datetime} <=> $b->{datetime} } @{ $self->{results} };
 
+	# wings (different departures which are coupled as one train) contain
+	# references to each other. therefore, they must be processed last.
+	$self->create_wing_refs;
+
+	# same goes for replacement refs (the <ref> tag in the fchg document)
+	$self->create_replacement_refs;
+
 	return $self;
 }
 
@@ -128,8 +136,8 @@ sub add_result {
 		$data{route_pre}   = $e_ar->getAttribute('ppth');
 		$data{route_start} = $e_ar->getAttribute('pde');
 		$data{transfer}    = $e_ar->getAttribute('tra');
-		$data{arrival_wings} = $e_ar->getAttribute('wings');
-		$data{unk_ar_hi}     = $e_ar->getAttribute('hi');
+		$data{arrival_wing_ids} = $e_ar->getAttribute('wings');
+		$data{unk_ar_hi}        = $e_ar->getAttribute('hi');
 	}
 
 	if ($e_dp) {
@@ -138,8 +146,16 @@ sub add_result {
 		$data{route_post}   = $e_dp->getAttribute('ppth');
 		$data{route_end}    = $e_dp->getAttribute('pde');
 		$data{transfer}     = $e_dp->getAttribute('tra');
-		$data{departure_wings} = $e_dp->getAttribute('wings');
-		$data{unk_dp_hi}       = $e_dp->getAttribute('hi');
+		$data{departure_wing_ids} = $e_dp->getAttribute('wings');
+		$data{unk_dp_hi}          = $e_dp->getAttribute('hi');
+	}
+
+	if ( $data{arrival_wing_ids} ) {
+		$data{arrival_wing_ids} = [ split( /\|/, $data{arrival_wing_ids} ) ];
+	}
+	if ( $data{departure_wing_ids} ) {
+		$data{departure_wing_ids}
+		  = [ split( /\|/, $data{departure_wing_ids} ) ];
 	}
 
 	my $result = Travel::Status::DE::IRIS::Result->new(%data);
@@ -161,6 +177,11 @@ sub get_timetable {
 	my $res = $ua->get(
 		$dt->strftime( $self->{iris_base} . "/plan/${eva}/%y%m%d/%H" ) );
 
+	if ( $self->{developer_mode} ) {
+		say 'GET '
+		  . $dt->strftime( $self->{iris_base} . "/plan/${eva}/%y%m%d/%H" );
+	}
+
 	if ( $res->is_error ) {
 		$self->{warnstr} = 'Failed to fetch a schedule part: Server returned '
 		  . $res->status_line;
@@ -169,8 +190,6 @@ sub get_timetable {
 
 	my $xml = XML::LibXML->load_xml( string => $res->decoded_content );
 
-	#say $xml->toString(1);
-
 	my $station = ( $xml->findnodes('/timetable') )[0]->getAttribute('station');
 
 	for my $s ( $xml->findnodes('/timetable/s') ) {
@@ -187,6 +206,10 @@ sub get_realtime {
 	my $eva = $self->{nodes}{station}->getAttribute('eva');
 	my $res = $self->{user_agent}->get( $self->{iris_base} . "/fchg/${eva}" );
 
+	if ( $self->{developer_mode} ) {
+		say 'GET ' . $self->{iris_base} . "/fchg/${eva}";
+	}
+
 	if ( $res->is_error ) {
 		$self->{warnstr} = 'Failed to fetch realtime data: Server returned '
 		  . $res->status_line;
@@ -198,12 +221,12 @@ sub get_realtime {
 	my $station = ( $xml->findnodes('/timetable') )[0]->getAttribute('station');
 
 	for my $s ( $xml->findnodes('/timetable/s') ) {
-		my $id    = $s->getAttribute('id');
-		my $e_tl  = ( $s->findnodes('./tl') )[0];
-		my $e_ar  = ( $s->findnodes('./ar') )[0];
-		my $e_dp  = ( $s->findnodes('./dp') )[0];
-		my $e_ref = ( $s->findnodes('./ref') )[0];
-		my @e_ms  = $s->findnodes('.//m');
+		my $id     = $s->getAttribute('id');
+		my $e_tl   = ( $s->findnodes('./tl') )[0];
+		my $e_ar   = ( $s->findnodes('./ar') )[0];
+		my $e_dp   = ( $s->findnodes('./dp') )[0];
+		my @e_refs = $s->findnodes('./ref/tl');
+		my @e_ms   = $s->findnodes('.//m');
 
 		my %messages;
 
@@ -211,6 +234,9 @@ sub get_realtime {
 
 		if ( not $result ) {
 			$result = $self->add_result( $station, $s );
+			if ($result) {
+				$result->set_unscheduled(1);
+			}
 		}
 		if ( not $result ) {
 			next;
@@ -245,8 +271,8 @@ sub get_realtime {
 				unknown_o => $e_tl->getAttribute('o'),    # owner: 03/80/R2/...
 			);
 		}
-		if ($e_ref) {
-			$result->set_ref(
+		for my $e_ref (@e_refs) {
+			$result->add_raw_ref(
 				class     => $e_ref->getAttribute('f'),    # D N S F
 				unknown_t => $e_ref->getAttribute('t'),    # p
 				train_no  => $e_ref->getAttribute('n'),    # dep number
@@ -258,7 +284,7 @@ sub get_realtime {
 		if ($e_ar) {
 			$result->set_ar(
 				arrival_ts      => $e_ar->getAttribute('ct'),
-				plan_arrivaL_ts => $e_ar->getAttribute('pt'),
+				plan_arrival_ts => $e_ar->getAttribute('pt'),
 				platform        => $e_ar->getAttribute('cp'),
 				route_pre       => $e_ar->getAttribute('cpth'),
 				sched_route_pre => $e_ar->getAttribute('ppth'),
@@ -282,6 +308,59 @@ sub get_realtime {
 	return $self;
 }
 
+sub get_result_by_id {
+	my ( $self, $id ) = @_;
+
+	my $res = first { $_->wing_id eq $id } @{ $self->{results} };
+	return $res;
+}
+
+sub get_result_by_train {
+	my ( $self, $type, $train_no ) = @_;
+
+	my $res = first { $_->type eq $type and $_->train_no eq $train_no }
+	@{ $self->{results} };
+	return $res;
+}
+
+sub create_wing_refs {
+	my ($self) = @_;
+
+	for my $r ( $self->results ) {
+		if ( $r->{departure_wing_ids} ) {
+			for my $wing_id ( @{ $r->{departure_wing_ids} } ) {
+				my $wingref = $self->get_result_by_id($wing_id);
+				if ($wingref) {
+					$r->add_departure_wingref($wingref);
+				}
+			}
+		}
+		if ( $r->{arrival_wing_ids} ) {
+			for my $wing_id ( @{ $r->{arrival_wing_ids} } ) {
+				my $wingref = $self->get_result_by_id($wing_id);
+				if ($wingref) {
+					$r->add_arrival_wingref($wingref);
+				}
+			}
+		}
+	}
+
+}
+
+sub create_replacement_refs {
+	my ($self) = @_;
+
+	for my $r ( $self->results ) {
+		for my $ref_hash ( @{ $r->{refs} // [] } ) {
+			my $ref = $self->get_result_by_train( $ref_hash->{type},
+				$ref_hash->{train_no} );
+			if ($ref) {
+				$r->add_reference($ref);
+			}
+		}
+	}
+}
+
 sub errstr {
 	my ($self) = @_;
 
@@ -327,7 +406,7 @@ Travel::Status::DE::IRIS - Interface to IRIS based web departure monitors.
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 DESCRIPTION
 
