@@ -13,8 +13,9 @@ use DateTime;
 use DateTime::Format::Strptime;
 use List::Compare;
 use List::MoreUtils qw(none uniq firstval);
+use Scalar::Util qw(weaken);
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 my %translation = (
 	2  => 'Polizeiliche Ermittlung',
@@ -95,10 +96,11 @@ my %translation = (
 
 Travel::Status::DE::IRIS::Result->mk_ro_accessors(
 	qw(arrival classes date datetime delay departure is_cancelled is_transfer
-	  line_no train_no_transfer old_train_id old_train_no platform raw_id
-	  realtime_xml route_start route_end sched_arrival sched_departure
-	  sched_platform sched_route_start sched_route_end start stop_no time
-	  train_id train_no transfer type unknown_t unknown_o)
+	  is_unscheduled is_wing line_no old_train_id old_train_no platform raw_id
+	  realtime_xml route_start route_end
+	  sched_arrival sched_departure sched_platform sched_route_start
+	  sched_route_end start stop_no time train_id train_no transfer type
+	  unknown_t unknown_o wing_id)
 );
 
 sub new {
@@ -113,6 +115,8 @@ sub new {
 
 	my ( $train_id, $start_ts, $stop_no ) = split( /.\K-/, $opt{raw_id} );
 
+	$ref->{wing_id} = "${train_id}-${start_ts}";
+	$ref->{is_wing} = 0;
 	$train_id =~ s{^-}{};
 
 	$ref->{start} = $strp->parse_datetime($start_ts);
@@ -305,12 +309,10 @@ sub set_realtime {
 	return $self;
 }
 
-sub set_ref {
+sub add_raw_ref {
 	my ( $self, %attrib ) = @_;
 
-	$self->{train_no_transfer} = $attrib{train_no};
-
-	# TODO
+	push( @{ $self->{refs} }, \%attrib );
 
 	return $self;
 }
@@ -323,6 +325,48 @@ sub set_tl {
 	return $self;
 }
 
+sub set_unscheduled {
+	my ( $self, $unscheduled ) = @_;
+
+	$self->{is_unscheduled} = $unscheduled;
+}
+
+sub add_arrival_wingref {
+	my ( $self, $ref ) = @_;
+
+	$ref->{is_wing} = 1;
+	weaken($ref);
+	push( @{ $self->{arrival_wings} }, $ref );
+	return $self;
+}
+
+sub add_departure_wingref {
+	my ( $self, $ref ) = @_;
+
+	$ref->{is_wing} = 1;
+	weaken($ref);
+	push( @{ $self->{departure_wings} }, $ref );
+	return $self;
+}
+
+sub add_reference {
+	my ( $self, $ref ) = @_;
+
+	$ref->add_inverse_reference($self);
+	weaken($ref);
+	push( @{ $self->{replacement_for} }, $ref );
+	return $self;
+}
+
+# never called externally
+sub add_inverse_reference {
+	my ( $self, $ref ) = @_;
+
+	weaken($ref);
+	push( @{ $self->{replaced_by} }, $ref );
+	return $self;
+}
+
 # List::Compare does not keep the order of its arguments (even with unsorted).
 # So we need to re-sort all stops to maintain their original order.
 sub sorted_sublist {
@@ -434,6 +478,42 @@ sub delay_messages {
 	return @ret;
 }
 
+sub arrival_wings {
+	my ($self) = @_;
+
+	if ( $self->{arrival_wings} ) {
+		return @{ $self->{arrival_wings} };
+	}
+	return;
+}
+
+sub departure_wings {
+	my ($self) = @_;
+
+	if ( $self->{departure_wings} ) {
+		return @{ $self->{departure_wings} };
+	}
+	return;
+}
+
+sub replaced_by {
+	my ($self) = @_;
+
+	if ( $self->{replaced_by} ) {
+		return @{ $self->{replaced_by} };
+	}
+	return;
+}
+
+sub replacement_for {
+	my ($self) = @_;
+
+	if ( $self->{replacement_for} ) {
+		return @{ $self->{replacement_for} };
+	}
+	return;
+}
+
 sub dump_message_codes {
 	my ($self) = @_;
 
@@ -672,7 +752,7 @@ arrival/departure received by Travel::Status::DE::IRIS
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 DESCRIPTION
 
@@ -697,6 +777,12 @@ set of actual stops (B<route_post>) minus the set of scheduled stops
 DateTime(3pm) object for the arrival date and time. undef if the
 train starts here. Contains realtime data if available.
 
+=item $result->arrival_wings
+
+Returns a list of references to Travel::Status::DE::IRIS::Result(3pm) objects
+which are coupled to this train on arrival. Returns nothing (false / empty list)
+otherwise.
+
 =item $result->canceled_stops
 
 Returns stops which are scheduled, but will not be served by this train.
@@ -744,6 +830,12 @@ most recent record will be returned.
 DateTime(3pm) object for the departure date and time. undef if the train ends
 here. Contains realtime data if available.
 
+=item $result->departure_wings
+
+Returns a list of references to Travel::Status::DE::IRIS::Result(3pm) objects
+which are coupled to this train on departure. Returns nothing (false / empty
+list) otherwise.
+
 =item $result->destination
 
 Alias for route_end.
@@ -772,6 +864,20 @@ the results and indicates it by setting B<is_transfer> to a true value.
 In case of a transfer, B<train_id> and B<train_no> are set to the "new"
 value, the old ones are available in B<old_train_id> and B<old_train_no>.
 
+=item $result->is_unscheduled
+
+True if the train does not appear in the requested plans. This can happen
+because of two reasons: Either the scheduled time and the actual time are so
+far apart that it should've arrived/departed long ago, or it really is an
+unscheduled train. In that case, it can be a replacement or an additional
+train. There is no logic to distinguish these cases yet.
+
+=item $result->is_wing
+
+Returns true if this result is a wing, false otherwise.
+A wing is a train which has its own ID and destination, but is currently
+coupled to another train and shares all or some of its route.
+
 =item $result->line
 
 Train type with line (such as C<< S 1 >>) if available, type with number
@@ -844,6 +950,18 @@ available.
 
 This is a developer option. It may be removed without prior warning.
 
+=item $result->replaced_by
+
+Returns a list of references to Travel::Status::DE::IRIS::Result(3pm) objects
+which replace the (usually cancelled) arrival/departure of this train.
+Returns nothing (false / empty list) otherwise.
+
+=item $result->replacement_for
+
+Returns a list of references to Travel::Status::DE::IRIS::Result(3pm) objects
+which this (usually unplanned) train is meant to replace.
+Returns nothing (false / empty list) otherwise.
+
 =item $result->route
 
 List of all stations served by this train, according to its schedule. Does
@@ -938,15 +1056,6 @@ year.  Note that it may change during the yearly itinerary update in december.
 
 Number of this train, unique per day. E.g. C<< 2225 >> for C<< IC 2225 >>.
 
-=item $result->train_no_transfer
-
-Number of this train after a following transfer, undefined if no such transfer
-exists. See B<is_transfer> for a note about this.
-
-Note that unlike B<old_train_no>, this information is always based on realtime
-data (not included in any schedule) and only set for stations before the
-transfer station, not the transfer station itself.
-
 =item $result->type
 
 Type of this train, e.g. C<< S >> for S-Bahn, C<< RE >> for Regional-Express,
